@@ -15,6 +15,7 @@ enum Playertypes
 {
 	STATUS_GUEST,
 	STATUS_REGISTERED,
+	STATUS_HOST,
 	STATUS_ADMIN,
 	STATUS_OWNER
 };
@@ -41,6 +42,7 @@ Player *guru_player_lookup(const char *name);
 void guru_player_save(Player *p);
 void guru_player_free(Player *p);
 Player *guru_player_new(void);
+void guru_player_cleanup(void);
 
 #endif
 
